CREATE OR REPLACE FUNCTION CALCULA_COMPENSACOES(
   P_CDEMPRESA      IN EMPRESA.CDEMPRESA%TYPE,
   P_CDIMPOSTONFE   IN MAPAAPURAC.CDIMPOSTO%TYPE,
   P_NRSEQUMAPAPU   IN MAPAAPURAC.NRSEQUMAPAPU%TYPE,
   P_DTINICIAL      IN MAPAAPURAC.DTINIPERIODO%TYPE,
   P_DTFINAL        IN MAPAAPURAC.DTINIPERIODO%TYPE,
   P_IDCUMUPISCOFIL IN FILIAL.IDCUMUPISCOFIL%TYPE) RETURN NUMBER
AS
   V_IDFORMAPUR        VARCHAR2(1);
   V_IDNFFORAPERAPU    VARCHAR2(1);
   V_DTINIFORPER       DATE;
   V_SEQITCOMP         VARCHAR2(4);
   V_VRCOMPENSACOES    NUMBER;
   V_IDIMPORECUPNFE    VARCHAR2(1);
   V_SEQCOMPENS        VARCHAR2(2);
   V_IDNFFORAPERAPUIMP VARCHAR2(1);
BEGIN
   V_VRCOMPENSACOES := 0;

   /******* Seleciona a forma de apurao do imposto retido na fonte e se o mesmo  ou no recupervel,
            e se utiliza vincula nf fora do perodo por imposto *******/
   SELECT MAX(IDFORMAPUR), MAX(IDIMPORECUP), MAX(IDNFFORAPERAPU)
     INTO V_IDFORMAPUR, V_IDIMPORECUPNFE, V_IDNFFORAPERAPUIMP
     FROM IMPOSTO I
    WHERE I.CDIMPOSTONFE = P_CDIMPOSTONFE;

   /******* Grava NULL no campo NRSEQMAPOIMP para as notas do perdodo informado, devido a reapurao de mapas *******/
   UPDATE IMPOSTONF
      SET NRSEQMAPOIMP = NULL
    WHERE NRSEQMAPOIMP = P_NRSEQUMAPAPU;

   /*Busca a Data Inicial para as notas fora do perodo e o parmetro para vincular NFS ao Mapa*/
   SELECT DTININFFORPER, NVL(IDNFFORAPERAPU,'N')
     INTO V_DTINIFORPER, V_IDNFFORAPERAPU
     FROM PARAMFISCAL;

   /*'I'  inutilizado, se usar o parmetro por imposto ele tem preferncia*/
   IF V_IDNFFORAPERAPUIMP <> 'I' THEN
      V_IDNFFORAPERAPU := V_IDNFFORAPERAPUIMP;
   END IF;

   /******* Update das notas de compensao. Nesse caso  atualizado o campo NRSEQMAPOIMP *******/
   /******* Na compensao so consideradas apenas notas de sada, pois se trata de compensao de valor retido por cliente *******/

   /*------------- Apurao do Imposto por Data de Competncia ou Data de Emisso (J que so consideradas apenas notas de sada)-------------*/
   IF (V_IDFORMAPUR = 'C') OR (V_IDFORMAPUR = 'E') THEN

      UPDATE IMPOSTONF
         SET NRSEQMAPOIMP = P_NRSEQUMAPAPU
       WHERE NRSEQMAPOIMP IS NULL
         AND (NRLANCTONF, CDFILIAL, CDIMPOSTO)  IN (SELECT IP.NRLANCTONF, IP.CDFILIAL, IP.CDIMPOSTO
                                                      FROM NOTAFISCAL N, FILIAL F, IMPOSTONF IP, IMPOSTO I
                                                     WHERE N.IDSTATUS      IN ('E','A')
                                                       AND N.CDEMPRESA     = P_CDEMPRESA
                                                       AND N.IDENTRSAIDOP  = 'S'
                                                       AND N.CDFILIEMIT    = F.CDFILIAL
                                                       AND N.CDFILIAL      = IP.CDFILIAL
                                                       AND N.NRLANCTONF    = IP.NRLANCTONF
                                                       AND I.CDIMPOSTONFE  = P_CDIMPOSTONFE
                                                       AND IP.CDIMPOSTO    = I.CDIMPOSTO
                                                       AND IP.NRSEQMAPOIMP IS NULL
                                                       AND ((F.IDCUMUPISCOFIL  = P_IDCUMUPISCOFIL) OR (P_IDCUMUPISCOFIL IS NULL))
                                                       AND (((V_IDNFFORAPERAPU = 'N') AND (N.DTEMISSAO BETWEEN V_DTINIFORPER AND P_DTFINAL)) OR
                                                            ((V_IDNFFORAPERAPU = 'S') AND (IP.DTREFMAPAPU IS NULL) AND (N.DTEMISSAO BETWEEN V_DTINIFORPER AND P_DTFINAL)) OR
                                                            ((V_IDNFFORAPERAPU = 'S') AND (IP.DTREFMAPAPU BETWEEN P_DTINICIAL AND P_DTFINAL) AND (N.DTEMISSAO BETWEEN V_DTINIFORPER AND P_DTFINAL)))
                                                       AND N.IDSISTEMA <> 'PAG'
                                                       AND N.IDSISTEMA <> 'REC'
                                                       AND N.IDSISTEMA <> 'FIS'
                                                       AND N.IDSISTEMA <> 'TMP');

   END IF;
  /*------------- Apurao do Imposto por Data de Entrada/Sada -------------*/
   IF V_IDFORMAPUR = 'D' THEN

      UPDATE IMPOSTONF
         SET NRSEQMAPOIMP = P_NRSEQUMAPAPU
       WHERE NRSEQMAPOIMP IS NULL
         AND (NRLANCTONF, CDFILIAL, CDIMPOSTO)  IN (SELECT IP.NRLANCTONF, IP.CDFILIAL, IP.CDIMPOSTO
                                                      FROM NOTAFISCAL N, FILIAL F, IMPOSTONF IP, IMPOSTO I
                                                     WHERE N.IDSTATUS      IN ('E','A')
                                                       AND N.CDEMPRESA     = P_CDEMPRESA
                                                       AND N.IDENTRSAIDOP  = 'S'
                                                       AND N.CDFILIEMIT    = F.CDFILIAL
                                                       AND N.CDFILIAL      = IP.CDFILIAL
                                                       AND N.NRLANCTONF    = IP.NRLANCTONF
                                                       AND I.CDIMPOSTONFE  = P_CDIMPOSTONFE
                                                       AND IP.CDIMPOSTO    = I.CDIMPOSTO
                                                       AND IP.NRSEQMAPOIMP IS NULL
                                                       AND ((F.IDCUMUPISCOFIL = P_IDCUMUPISCOFIL) OR (P_IDCUMUPISCOFIL IS NULL))
                                                       AND (((V_IDNFFORAPERAPU='N') AND (N.DTENTRSAID BETWEEN V_DTINIFORPER AND P_DTFINAL)) OR
                                                            ((V_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU IS NULL) AND (N.DTENTRSAID BETWEEN V_DTINIFORPER AND P_DTFINAL)) OR
                                                            ((V_IDNFFORAPERAPU='S') AND (IP.DTREFMAPAPU BETWEEN P_DTINICIAL AND P_DTFINAL) AND (N.DTENTRSAID BETWEEN V_DTINIFORPER AND P_DTFINAL)))
                                                       AND N.IDSISTEMA <> 'PAG'
                                                       AND N.IDSISTEMA <> 'REC'
                                                       AND N.IDSISTEMA <> 'FIS'
                                                       AND N.IDSISTEMA <> 'TMP');

   END IF;

  /*------------- Apurao do Imposto por Data de Pagamento -------------*/
   IF V_IDFORMAPUR = 'P' THEN

      UPDATE IMPOSTONF
         SET NRSEQMAPOIMP = P_NRSEQUMAPAPU
       WHERE NRSEQMAPOIMP IS NULL
         AND (NRLANCTONF, CDFILIAL, CDIMPOSTO) IN (SELECT IP.NRLANCTONF, IP.CDFILIAL, IP.CDIMPOSTO
                                                     FROM NOTAFISCAL N, FILIAL F, IMPOSTONF IP, IMPOSTO I,
                                                         (SELECT DISTINCT C.CDFILIAL, C.NRLANCTONF, BX.DTBAIXAREC
                                                          FROM CONTARECEBER C, BOLETRECEBER B, BAIXARECEBIDA BX
                                                          WHERE (C.CDFILIBOLREC = B.CDFILIAL)    AND
                                                                (C.NRLANBOLREC  = B.NRLANBOLREC) AND
                                                                (B.CDFILIBAIREC = BX.CDFILIAL)    AND
                                                                (B.NRBAIXAREC   = BX.NRBAIXAREC)  AND
                                                                (BX.CDEMPRESA   = P_CDEMPRESA)) A
                                                    WHERE N.CDEMPRESA         = P_CDEMPRESA
                                                      AND N.IDSTATUS          IN ('A','E')
                                                      AND N.IDENTRSAIDOP      = 'S'
                                                      AND N.CDFILIEMIT        = F.CDFILIAL
                                                      AND ((F.IDCUMUPISCOFIL  = P_IDCUMUPISCOFIL) OR (P_IDCUMUPISCOFIL IS NULL))
                                                      AND IP.CDFILIAL         = N.CDFILIAL
                                                      AND IP.NRLANCTONF       = N.NRLANCTONF
                                                      AND I.CDIMPOSTONFE      = P_CDIMPOSTONFE
                                                      AND IP.CDIMPOSTO        = I.CDIMPOSTO
                                                      AND IP.NRSEQMAPOIMP     IS NULL
                                                      AND N.CDFILIAL          = A.CDFILIAL
                                                      AND N.NRLANCTONF        = A.NRLANCTONF
                                                      AND (((V_IDNFFORAPERAPU = 'N') AND (A.DTBAIXAREC BETWEEN V_DTINIFORPER AND P_DTFINAL)) OR
                                                           ((V_IDNFFORAPERAPU = 'S') AND (IP.DTREFMAPAPU IS NULL) AND (A.DTBAIXAREC BETWEEN V_DTINIFORPER AND P_DTFINAL)) OR
                                                           ((V_IDNFFORAPERAPU = 'S') AND (IP.DTREFMAPAPU BETWEEN P_DTINICIAL AND P_DTFINAL) AND (A.DTBAIXAREC BETWEEN V_DTINIFORPER AND P_DTFINAL)))
                                                      AND N.IDSISTEMA <> 'PAG'
                                                      AND N.IDSISTEMA <> 'REC'
                                                      AND N.IDSISTEMA <> 'FIS'
                                                      AND N.IDSISTEMA <> 'TMP');

   END IF;

   /*------------- Gera Itens de Compensao -------------*/
   V_VRCOMPENSACOES := 0;
   IF P_IDCUMUPISCOFIL = 'S' THEN
      V_SEQCOMPENS := 'CR';
   ELSE
      IF P_IDCUMUPISCOFIL = 'N' THEN
         V_SEQCOMPENS := 'CP';
      ELSE
         V_SEQCOMPENS := 'CO';
      END IF;
   END IF;

   FOR C_COMP IN (SELECT I.CDFILIAL, I.NRLANCTONF, N.NRNOTAFISC, N.VRNOTAFISC,
                         DECODE(N.CDCLIENTE,NULL,DECODE(N.CDFORNECED,NULL,L.NRINSJURFILI,F.NRINSJURFORN),C.NRINSJURCLIE) AS INSCPART,
                         DECODE(N.CDCLIENTE,NULL,DECODE(N.CDFORNECED,NULL,L.NMFILIAL,F.NMFANTFORN),C.NMFANTCLIE) AS NOMEPART,
                         SUM(DECODE(V_IDIMPORECUPNFE,'S', DECODE(CDFISCALNF,'1',VRIMPOSTONF,0), VRIMPOSTONF)) AS VRCOMP
                    FROM IMPOSTONF I, NOTAFISCAL N, CLIENTE C, FORNECEDOR F, FILIAL L
                   WHERE I.NRSEQMAPOIMP = P_NRSEQUMAPAPU
                     AND I.CDFILIAL     = N.CDFILIAL
                     AND I.NRLANCTONF   = N.NRLANCTONF
                     AND N.IDENTRSAIDOP = 'S'
                     AND N.CDCLIENTE    = C.CDCLIENTE(+)
                     AND N.CDFORNECED   = F.CDFORNECED(+)
                     AND N.CDFILIRECE   = L.CDFILIAL(+)
                   GROUP BY I.CDFILIAL, I.NRLANCTONF, N.NRNOTAFISC, N.VRNOTAFISC,
                            DECODE(N.CDCLIENTE,NULL,DECODE(N.CDFORNECED,NULL,L.NRINSJURFILI,F.NRINSJURFORN),C.NRINSJURCLIE),
                            DECODE(N.CDCLIENTE,NULL,DECODE(N.CDFORNECED,NULL,L.NMFILIAL,F.NMFANTFORN),C.NMFANTCLIE)) LOOP

      /******* Verifica se j existe sequencial de compensao para a nota *******/
      SELECT MIN(NRSEQITDETAPU)
        INTO V_SEQITCOMP
        FROM ITDETAPUCONTR
       WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
         AND IDTIPODETAPU = V_SEQCOMPENS
         AND CDFILIAL     = C_COMP.CDFILIAL
         AND NRLANCTONF   = C_COMP.NRLANCTONF;

      IF V_SEQITCOMP IS NOT NULL THEN
         /******* Apaga o registro da nota de compensao *******/
         DELETE FROM ITDETAPUCONTR
          WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
            AND IDTIPODETAPU = V_SEQCOMPENS
            AND CDFILIAL     = C_COMP.CDFILIAL
            AND NRLANCTONF   = C_COMP.NRLANCTONF;
      END IF;

      IF C_COMP.VRCOMP > 0 THEN
         IF V_SEQITCOMP IS NULL THEN
            /******* Gera o prximo sequencial de compensaes caso no existisse sequencial para a nota *******/
            SELECT MAX(NRSEQITDETAPU)+1
              INTO V_SEQITCOMP
              FROM ITDETAPUCONTR
             WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
               AND IDTIPODETAPU = V_SEQCOMPENS;
         END IF;

         /******* Insere o item de compensao *******/
         INSERT INTO ITDETAPUCONTR
            (NRSEQUMAPAPU, IDTIPODETAPU, NRSEQITDETAPU,
             DSITEMDETAPU,
             VRITEMDETAPU, CDFILIAL, NRLANCTONF)
         VALUES
            (P_NRSEQUMAPAPU, V_SEQCOMPENS, LPAD(NVL(V_SEQITCOMP,1),4,'0'),
             C_COMP.INSCPART||' - '||C_COMP.NOMEPART||' (VALOR NF: '||C_COMP.VRNOTAFISC||')',
             C_COMP.VRCOMP, C_COMP.CDFILIAL, C_COMP.NRLANCTONF);
      END IF;
   END LOOP;

   /******* Apaga os itens referentes a notas que no possuem mais o imposto *******/
   DELETE FROM ITDETAPUCONTR
    WHERE NRLANCTONF IS NOT NULL
	   AND NRSEQUMAPAPU = P_NRSEQUMAPAPU
	   AND (CDFILIAL, NRLANCTONF) NOT IN (SELECT CDFILIAL, NRLANCTONF
	                                        FROM IMPOSTONF
	   									           WHERE NRSEQMAPOIMP = P_NRSEQUMAPAPU);

   /******* Busca o valor total das compensaes *******/
   SELECT NVL(SUM(VRITEMDETAPU),0)
     INTO V_VRCOMPENSACOES
     FROM ITDETAPUCONTR I
    WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
      AND IDTIPODETAPU = V_SEQCOMPENS;

   RETURN(V_VRCOMPENSACOES);
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CALCULA_COMPENSACOES','5.65.000'); END;
/
